#pragma comment(lib,"dinput8.lib")
#pragma comment(lib,"dxguid.lib")
#include <dinput.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <vector>
#include "GMaker.h"

#define GM_export extern "C" __declspec (dllexport)

LPDIRECTINPUT8  DInput; 
HWND hWnd;
std::vector<IDirectInputDevice8*> mice;

GMPROC* pds_list_add;
double ds_list_add(double id,GMVariable val)
{
	GMVariable args[] = {id,val};
	return GM::ExternalCall(pds_list_add, 2, 2, args);
}

BOOL CALLBACK EnumMice(const DIDEVICEINSTANCE* inst, VOID* pContext)
{
	IDirectInputDevice8* tdev;
	if(DInput->CreateDevice(inst->guidInstance, &tdev,NULL)<0)
		return DIENUM_CONTINUE;

	if(tdev->SetDataFormat(&c_dfDIMouse2)<0)
	{
		tdev->Release();
		return DIENUM_CONTINUE;
	}
	if(tdev->SetCooperativeLevel(hWnd,DISCL_BACKGROUND | DISCL_NONEXCLUSIVE))
	{
		tdev->Release();
		return DIENUM_CONTINUE;
	}

	DIPROPDWORD tprop;
	tprop.diph.dwSize=sizeof(DIPROPDWORD);
	tprop.diph.dwHeaderSize=sizeof(DIPROPHEADER);
	tprop.diph.dwHow=DIPH_DEVICE;
	tprop.diph.dwObj=0;
	tprop.dwData=DIPROPAXISMODE_REL;
	tdev->SetProperty(DIPROP_AXISMODE ,&tprop.diph);
	tprop.diph.dwSize=sizeof(DIPROPDWORD);
	tprop.diph.dwHeaderSize=sizeof(DIPROPHEADER);
	tprop.diph.dwHow=DIPH_DEVICE;
	tprop.diph.dwObj=0;
	tprop.dwData=0;
	tdev->SetProperty(DIPROP_BUFFERSIZE,&tprop.diph);

	tdev->Acquire();

	mice.push_back(tdev);
	return DIENUM_CONTINUE;
}

GM_export double InitDIM(double hwnd)
{
	hWnd=(HWND)(DWORD)hwnd;
	pds_list_add=(GMPROC*)GM::GetProcAddress("ds_list_add");
	if(DirectInput8Create((HINSTANCE)GetWindowLong(hWnd,GWL_HINSTANCE),DIRECTINPUT_VERSION,IID_IDirectInput8, (void**)&DInput, NULL)<0) 
	{
		return -1;
	} 

	DInput->EnumDevices(DI8DEVCLASS_POINTER,EnumMice,NULL,DIEDFL_ATTACHEDONLY);

	return mice.size();
}
GM_export double DeinitDIM()
{
	while(mice.size())
	{
		mice.back()->Unacquire();
		mice.back()->Release();
		mice.pop_back();
	}
	DInput->Release();
	return 0;
}
GM_export double GetDIMInput(double ListID)
{
	for(unsigned int i=0;i<mice.size();i++)
	{
		DIMOUSESTATE2 state;
		if(mice[i]->Poll()!=DI_OK)
		{
			mice[i]->Acquire(); //If the device can't be polled, try reaquiring it.
			mice[i]->Poll();
		}
		if(mice[i]->GetDeviceState(sizeof(state),&state)!=DI_OK)
		{
			ZeroMemory(&state,sizeof(state)); //If the input cannot be gotten, fille the structure with 0
											  //From GM, this will look like the buttons are all unpressed, 
											  //and it didn't move. Hopefully the mouse can be reaquired next step.
		}
		ds_list_add(ListID,double(state.lX));
		ds_list_add(ListID,double(state.lY));
		ds_list_add(ListID,double(state.lZ));
		for(int j=0;j<8;j++)
		{
			ds_list_add(ListID,double(state.rgbButtons[j]));
		}
	}
	return 0;
}